@@ -1,15 +1,17 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python3
 
-import sys, os, commands
+import sys, os
 import gettext
 import rfkillMagic
+import setproctitle
 import subprocess
-from BlueberrySettingsWidgets import SettingsPage, SettingsBox, SettingsRow
+from BlueberrySettingsWidgets import SettingsPage, SettingsRow
 import gi
 gi.require_version('Gtk', '3.0')
 gi.require_version('GnomeBluetooth', '1.0')
 from gi.repository import Gtk, GnomeBluetooth, Gio
 
+BLUETOOTH_RFKILL_ERR         = "rfkill-err"
 BLUETOOTH_DISABLED_PAGE      = "disabled-page"
 BLUETOOTH_HW_DISABLED_PAGE   = "hw-disabled-page"
 BLUETOOTH_NO_DEVICES_PAGE    = "no-devices-page"
@@ -18,6 +20,8 @@ BLUETOOTH_WORKING_PAGE       = "working-page"
 # i18n
 gettext.install("blueberry", "/usr/share/locale")
 
+setproctitle.setproctitle("blueberry")
+
 class Blueberry(Gtk.Application):
     ''' Create the UI '''
     def __init__(self):
@@ -36,7 +40,7 @@ class Blueberry(Gtk.Application):
             self.create_window()
 
     def detect_desktop_environment(self):
-        wm_info = commands.getoutput("wmctrl -m")
+        wm_info = subprocess.getoutput("wmctrl -m")
         if "XDG_CURRENT_DESKTOP" in os.environ:
             xdg_current_desktop = os.environ["XDG_CURRENT_DESKTOP"]
         else:
@@ -66,7 +70,7 @@ class Blueberry(Gtk.Application):
             self.configuration_tools = {"sound": "pavucontrol", "keyboard": "lxinput", "mouse": "lxinput"}
         else:
             self.de = "Unknown"
-            print "Warning: DE could not be detected!"
+            print("Warning: DE could not be detected!")
             self.configuration_tools = {}
             if os.path.exists("/usr/bin/pavucontrol"):
                 self.configuration_tools["sound"] = "pavucontrol"
@@ -122,6 +126,7 @@ class Blueberry(Gtk.Application):
         self.status_image.show()
 
         self.stack = Gtk.Stack()
+        self.error_label = self.add_stack_page(_("An error has occurred"), BLUETOOTH_RFKILL_ERR);
         self.add_stack_page(_("Bluetooth is disabled"), BLUETOOTH_DISABLED_PAGE);
         self.add_stack_page(_("No Bluetooth adapters found"), BLUETOOTH_NO_DEVICES_PAGE);
         self.add_stack_page(_("Bluetooth is disabled by hardware switch"), BLUETOOTH_HW_DISABLED_PAGE);
@@ -154,7 +159,7 @@ class Blueberry(Gtk.Application):
         if adapter_name is not None:
             self.adapter_name_entry.set_text(adapter_name)
         self.adapter_name_entry.connect("changed", self.on_adapter_name_changed)
-        row = SettingsRow(Gtk.Label(_("Name")), self.adapter_name_entry)
+        row = SettingsRow(Gtk.Label(label=_("Name")), self.adapter_name_entry)
         row.set_tooltip_text(_("This is the Bluetooth name of your computer"))
         section.add_row(row)
 
@@ -192,7 +197,7 @@ class Blueberry(Gtk.Application):
 
     def panel_changed(self, widget, panel):
         if not panel in self.configuration_tools:
-            print "Warning, no configuration tool known for panel '%s'" % panel
+            print("Warning, no configuration tool known for panel '%s'" % panel)
         else:
             os.system("%s &" % self.configuration_tools[panel])
 
@@ -224,11 +229,12 @@ class Blueberry(Gtk.Application):
         label = Gtk.Label(label=message)
         self.stack.add_named(label, name)
         label.show()
+        return label
 
     def get_default_adapter_name(self):
         name = None
         try:
-            output = subprocess.check_output(["timeout", "2s", "bt-adapter", "-i"]).strip()
+            output = subprocess.check_output(["timeout", "2s", "bt-adapter", "-i"]).decode("utf-8").strip()
             for line in output.split("\n"):
                 line = line.strip()
                 if line.startswith("Alias: "):
@@ -248,6 +254,9 @@ class Blueberry(Gtk.Application):
             scrolledwindow.set_shadow_type(Gtk.ShadowType.NONE)
             viewport = scrolledwindow.get_children()[0]
             vbox = viewport.get_children()[0]
+            spinner = vbox.get_children()[1].get_children()[0].get_children()[1]
+            if spinner.props.active:
+                spinner.stop()
             explanation_label = vbox.get_children()[0]
             name = self.get_default_adapter_name()
             if name is not None:
@@ -260,7 +269,7 @@ class Blueberry(Gtk.Application):
             else:
                 explanation_label.set_label("")
         except Exception as e:
-            print (e)
+            print(e)
             return None
 
     def update_ui_callback(self):
@@ -268,7 +277,14 @@ class Blueberry(Gtk.Application):
         sensitive = False
         page = ""
 
-        if not self.rfkill.have_adapter:
+        if self.rfkill.rfkill_err is not None:
+            self.error_label.set_markup("%s\n%s" % (_("An error has occurred"), self.rfkill.rfkill_err))
+            page = BLUETOOTH_RFKILL_ERR
+            sensitive = True
+            # Set Switch back to Off position
+            self.rf_switch.set_active(False)
+            self.status_image.set_from_icon_name("blueberry-disabled", Gtk.IconSize.DIALOG)
+        elif not self.rfkill.have_adapter:
             page = BLUETOOTH_NO_DEVICES_PAGE
             self.status_image.set_from_icon_name("blueberry-disabled", Gtk.IconSize.DIALOG)
         elif self.rfkill.hard_block:
